import io
import math
import traceback
import wave

import librosa


def get_audio_duration_bytes(byte_buffer: io.BytesIO):
    """
    获取音频的时长
    :param byte_buffer: 音频字节流
    :return:音频长度
    """
    try:
        if not byte_buffer:
            return 0
        byte_buffer.seek(0)
        with wave.open(byte_buffer, 'rb') as combined_audio:
            frame_rate = combined_audio.getframerate()
            num_frames = combined_audio.getnframes()
            duration = num_frames / float(frame_rate)
            return round(duration, 1)
    except (Exception,):
        traceback.print_exc()
        return 0


def get_audio_duration(file_path):
    try:
        duration = librosa.get_duration(filename=file_path)
        return ceil_to_2decimal(duration)
    except Exception as e:
        print(f"Error: {e}")
        return None


def ceil_to_2decimal(number):
    if not number:
        return None
    return math.ceil(number * 100) / 100


if __name__ == '__main__':
    rs = get_audio_duration('./voices/tvb_shot.wav')
    print(f"音频时长: {rs}秒")



import io
import os

from pydub import AudioSegment
from pydub.silence import detect_silence


def trim_silence_from_bytesio(byte_buffer, output_format='wav', silence_thresh=-50, chunk_size=100) -> io.BytesIO:
    if not byte_buffer:
        return byte_buffer
    byte_buffer.seek(0)
    audio = AudioSegment.from_file(byte_buffer, format=output_format)

    silence_ranges = detect_silence(audio, min_silence_len=chunk_size, silence_thresh=silence_thresh)
    trimmed_audio = audio
    if silence_ranges:
        start_trim_begin = silence_ranges[0][0]
        start_trim_end = silence_ranges[0][1]
        print('before length audio:', len(audio))
        print("start_trim_begin:", start_trim_begin, "start_trim_end:", start_trim_end)
        if start_trim_begin == 0:
            print('开头存在空白音')
            trim_begin = start_trim_end
        else:
            trim_begin = 0
        end_trim_begin = silence_ranges[-1][0]
        end_trim_end = silence_ranges[-1][1]  # 最后一段静音结束的时间
        print("end_trim_begin:", end_trim_begin, "end_trim_end:", end_trim_end)
        if end_trim_end == len(audio):
            print('结尾存在空白音')
            trim_end = end_trim_begin
        else:
            trim_end = len(audio)
        trimmed_audio = audio[trim_begin:trim_end]
        print('after length audio:', len(trimmed_audio))
    output_stream = io.BytesIO()
    trimmed_audio.export(output_stream, format=output_format)

    return output_stream


def trim_silence(input_path, output_path, silence_thresh=-90, chunk_size=100):
    """
    Detect and trim silence from audio file

    Args:
        input_path (str): Path to input audio file
        output_path (str): Path to save output audio file
        silence_thresh (int): Silence threshold in dB (default: -50)
        chunk_size (int): Minimum silence length in ms (default: 100)

    Returns:
        bool: True if processing successful, False otherwise
    """
    try:
        # Check if input file exists
        if not input_path or not os.path.exists(input_path):
            print(f"Input file does not exist: {input_path}")
            return False

        # Load audio from file
        audio = AudioSegment.from_file(input_path)

        # Detect silence ranges
        silence_ranges = detect_silence(audio, min_silence_len=chunk_size, silence_thresh=silence_thresh)

        trimmed_audio = audio
        if silence_ranges:
            start_trim_begin = silence_ranges[0][0]
            start_trim_end = silence_ranges[0][1]
            print('Before length audio:', len(audio))
            print("Start trim begin:", start_trim_begin, "start trim end:", start_trim_end)

            if start_trim_begin == 0:
                print('开头存在空白音 (Silence at beginning)')
                trim_begin = start_trim_end
            else:
                trim_begin = 0

            end_trim_begin = silence_ranges[-1][0]
            end_trim_end = silence_ranges[-1][1]  # 最后一段静音结束的时间
            print("End trim begin:", end_trim_begin, "end trim end:", end_trim_end)

            if end_trim_end == len(audio):
                print('结尾存在空白音 (Silence at end)')
                trim_end = end_trim_begin
            else:
                trim_end = len(audio)

            trimmed_audio = audio[trim_begin:trim_end]
            print('After length audio:', len(trimmed_audio))

        output_format = os.path.splitext(output_path)[1][1:].lower()  # Get format from file extension
        if not output_format:
            output_format = 'wav'  # Default format

        trimmed_audio.export(output_path, format=output_format)
        print(f"Audio saved to: {output_path}")
        return output_path

    except Exception as e:
        print(f"Error processing audio: {str(e)}")
        return None


if __name__ == '__main__':
    res = trim_silence("./voices/530.mp3", "./voices/530_after.mp3")
    print(res)



# -*- coding:utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='audioprep',
    version='1.0.9',
    description=(
        'AI audio preparation tools, including audio trimming, duration calculation, and subtitle correction.'
    ),
    keywords=(
        "audio, audio processing, audio trimming, audio duration, subtitle correction, AI tools"),
    long_description_content_type="text/markdown",
    long_description=open('README.md', encoding='utf-8').read(),
    author='abo123456789',
    author_email='abcdef123456chen@sohu.com',
    maintainer='abo123456789',
    maintainer_email='abcdef123456chen@sohu.com',
    license='MIT License',
    install_requires=[
        "pydub>=0.25.1",
        "librosa>=0.10.2",
        "wave>=0.0.2",
    ],
    packages=find_packages(),
    platforms=["all"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries'
    ])
